#region Using

using System;
using System.Web;
using System.IO;

#endregion

/// <summary>
/// Serves files from the disk asynchronously.
/// </summary>
public class AsyncFileHandler : IHttpAsyncHandler
{

	/// <summary>
	/// Serves the content asynchronously to the output stream.
	/// <remarks>
	/// You only have to modify this method.
	/// </remarks>
	/// </summary>
	private void ServeContent(HttpContext context)
	{
		string filePath = context.Server.MapPath(context.Request.QueryString.ToString());
		if (File.Exists(filePath))
		{
			context.Response.TransmitFile(filePath);
		}
		else
		{
			context.Response.Write("File does not exist");
			context.Response.StatusCode = 404;
			context.Response.End();
		}
	}

	#region IHttpAsyncHandler Members

	private AsyncProcessorDelegate _Delegate;
	protected delegate void AsyncProcessorDelegate(HttpContext context);

	/// <summary>
	/// Initiates an asynchronous call to the HTTP handler.
	/// </summary>
	/// <param name="context">An <see cref="T:System.Web.HttpContext"></see> object that provides references to intrinsic server objects (for example, Request, Response, Session, and Server) used to service HTTP requests.</param>
	/// <param name="cb">The <see cref="T:System.AsyncCallback"></see> to call when the asynchronous method call is complete. If cb is null, the delegate is not called.</param>
	/// <param name="extraData">Any extra data needed to process the request.</param>
	/// <returns>
	/// An <see cref="T:System.IAsyncResult"></see> that contains information about the status of the process.
	/// </returns>
	public IAsyncResult BeginProcessRequest(HttpContext context, AsyncCallback cb, object extraData)
	{
		_Delegate = new AsyncProcessorDelegate(ProcessRequest);
		return _Delegate.BeginInvoke(context, cb, extraData);
	}

	/// <summary>
	/// Provides an asynchronous process End method when the process ends.
	/// </summary>
	/// <param name="result">An <see cref="T:System.IAsyncResult"></see> that contains information about the status of the process.</param>
	public void EndProcessRequest(IAsyncResult result)
	{
		_Delegate.EndInvoke(result);
	}

	#endregion

	#region IHttpHandler Members

	/// <summary>
	/// Gets a value indicating whether another request can use the <see cref="T:System.Web.IHttpHandler"></see> instance.
	/// </summary>
	/// <value></value>
	/// <returns>true if the <see cref="T:System.Web.IHttpHandler"></see> instance is reusable; otherwise, false.</returns>
	public bool IsReusable
	{
		get { return true; }
	}

	/// <summary>
	/// Enables processing of HTTP Web requests by a custom HttpHandler that implements the <see cref="T:System.Web.IHttpHandler"></see> interface.
	/// </summary>
	/// <param name="context">An <see cref="T:System.Web.HttpContext"></see> object that provides references to the intrinsic server objects (for example, Request, Response, Session, and Server) used to service HTTP requests.</param>
	public void ProcessRequest(HttpContext context)
	{
		ServeContent(context);
	}

	#endregion
}